# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursorPaginationAfter, AsyncCursorPaginationAfter
from ...._base_client import AsyncPaginator, make_request_options
from ....types.durable_objects.namespaces import object_list_params
from ....types.durable_objects.namespaces.durable_object import DurableObject

__all__ = ["ObjectsResource", "AsyncObjectsResource"]


class ObjectsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ObjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ObjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ObjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ObjectsResourceWithStreamingResponse(self)

    def list(
        self,
        id: str,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        limit: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPaginationAfter[DurableObject]:
        """
        Returns the Durable Objects in a given namespace.

        Args:
          account_id: Identifier.

          id: ID of the namespace.

          cursor: Opaque token indicating the position from which to continue when requesting the
              next set of records. A valid value for the cursor can be obtained from the
              cursors object in the result_info structure.

          limit: The number of objects to return. The cursor attribute may be used to iterate
              over the next batch of objects if there are more than the limit.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/durable_objects/namespaces/{id}/objects", account_id=account_id, id=id
            ),
            page=SyncCursorPaginationAfter[DurableObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    object_list_params.ObjectListParams,
                ),
            ),
            model=DurableObject,
        )


class AsyncObjectsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncObjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncObjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncObjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncObjectsResourceWithStreamingResponse(self)

    def list(
        self,
        id: str,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        limit: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DurableObject, AsyncCursorPaginationAfter[DurableObject]]:
        """
        Returns the Durable Objects in a given namespace.

        Args:
          account_id: Identifier.

          id: ID of the namespace.

          cursor: Opaque token indicating the position from which to continue when requesting the
              next set of records. A valid value for the cursor can be obtained from the
              cursors object in the result_info structure.

          limit: The number of objects to return. The cursor attribute may be used to iterate
              over the next batch of objects if there are more than the limit.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/durable_objects/namespaces/{id}/objects", account_id=account_id, id=id
            ),
            page=AsyncCursorPaginationAfter[DurableObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    object_list_params.ObjectListParams,
                ),
            ),
            model=DurableObject,
        )


class ObjectsResourceWithRawResponse:
    def __init__(self, objects: ObjectsResource) -> None:
        self._objects = objects

        self.list = to_raw_response_wrapper(
            objects.list,
        )


class AsyncObjectsResourceWithRawResponse:
    def __init__(self, objects: AsyncObjectsResource) -> None:
        self._objects = objects

        self.list = async_to_raw_response_wrapper(
            objects.list,
        )


class ObjectsResourceWithStreamingResponse:
    def __init__(self, objects: ObjectsResource) -> None:
        self._objects = objects

        self.list = to_streamed_response_wrapper(
            objects.list,
        )


class AsyncObjectsResourceWithStreamingResponse:
    def __init__(self, objects: AsyncObjectsResource) -> None:
        self._objects = objects

        self.list = async_to_streamed_response_wrapper(
            objects.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["IPUpdateParams"]


class IPUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier of a Cloudflare account."""

    address_map_id: Required[str]
    """Identifier of an Address Map."""

    body: Required[object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .on_ramp_get_params import OnRampGetParams as OnRampGetParams
from .on_ramp_edit_params import OnRampEditParams as OnRampEditParams
from .on_ramp_list_params import OnRampListParams as OnRampListParams
from .resource_get_params import ResourceGetParams as ResourceGetParams
from .on_ramp_get_response import OnRampGetResponse as OnRampGetResponse
from .resource_list_params import ResourceListParams as ResourceListParams
from .on_ramp_create_params import OnRampCreateParams as OnRampCreateParams
from .on_ramp_delete_params import OnRampDeleteParams as OnRampDeleteParams
from .on_ramp_edit_response import OnRampEditResponse as OnRampEditResponse
from .on_ramp_list_response import OnRampListResponse as OnRampListResponse
from .on_ramp_plan_response import OnRampPlanResponse as OnRampPlanResponse
from .on_ramp_update_params import OnRampUpdateParams as OnRampUpdateParams
from .resource_get_response import ResourceGetResponse as ResourceGetResponse
from .on_ramp_apply_response import OnRampApplyResponse as OnRampApplyResponse
from .resource_export_params import ResourceExportParams as ResourceExportParams
from .resource_list_response import ResourceListResponse as ResourceListResponse
from .on_ramp_create_response import OnRampCreateResponse as OnRampCreateResponse
from .on_ramp_delete_response import OnRampDeleteResponse as OnRampDeleteResponse
from .on_ramp_update_response import OnRampUpdateResponse as OnRampUpdateResponse
from .catalog_sync_edit_params import CatalogSyncEditParams as CatalogSyncEditParams
from .catalog_sync_get_response import CatalogSyncGetResponse as CatalogSyncGetResponse
from .catalog_sync_create_params import CatalogSyncCreateParams as CatalogSyncCreateParams
from .catalog_sync_delete_params import CatalogSyncDeleteParams as CatalogSyncDeleteParams
from .catalog_sync_edit_response import CatalogSyncEditResponse as CatalogSyncEditResponse
from .catalog_sync_list_response import CatalogSyncListResponse as CatalogSyncListResponse
from .catalog_sync_update_params import CatalogSyncUpdateParams as CatalogSyncUpdateParams
from .catalog_sync_create_response import CatalogSyncCreateResponse as CatalogSyncCreateResponse
from .catalog_sync_delete_response import CatalogSyncDeleteResponse as CatalogSyncDeleteResponse
from .catalog_sync_update_response import CatalogSyncUpdateResponse as CatalogSyncUpdateResponse
from .cloud_integration_get_params import CloudIntegrationGetParams as CloudIntegrationGetParams
from .catalog_sync_refresh_response import CatalogSyncRefreshResponse as CatalogSyncRefreshResponse
from .cloud_integration_edit_params import CloudIntegrationEditParams as CloudIntegrationEditParams
from .cloud_integration_list_params import CloudIntegrationListParams as CloudIntegrationListParams
from .cloud_integration_get_response import CloudIntegrationGetResponse as CloudIntegrationGetResponse
from .resource_policy_preview_params import ResourcePolicyPreviewParams as ResourcePolicyPreviewParams
from .cloud_integration_create_params import CloudIntegrationCreateParams as CloudIntegrationCreateParams
from .cloud_integration_edit_response import CloudIntegrationEditResponse as CloudIntegrationEditResponse
from .cloud_integration_list_response import CloudIntegrationListResponse as CloudIntegrationListResponse
from .cloud_integration_update_params import CloudIntegrationUpdateParams as CloudIntegrationUpdateParams
from .resource_policy_preview_response import ResourcePolicyPreviewResponse as ResourcePolicyPreviewResponse
from .cloud_integration_create_response import CloudIntegrationCreateResponse as CloudIntegrationCreateResponse
from .cloud_integration_delete_response import CloudIntegrationDeleteResponse as CloudIntegrationDeleteResponse
from .cloud_integration_discover_params import CloudIntegrationDiscoverParams as CloudIntegrationDiscoverParams
from .cloud_integration_update_response import CloudIntegrationUpdateResponse as CloudIntegrationUpdateResponse
from .cloud_integration_discover_response import CloudIntegrationDiscoverResponse as CloudIntegrationDiscoverResponse
from .cloud_integration_discover_all_response import (
    CloudIntegrationDiscoverAllResponse as CloudIntegrationDiscoverAllResponse,
)
from .cloud_integration_initial_setup_response import (
    CloudIntegrationInitialSetupResponse as CloudIntegrationInitialSetupResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["AccessRuleDeleteResponse"]


class AccessRuleDeleteResponse(BaseModel):
    id: str
    """Defines an identifier."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..shared.token_value import TokenValue
from ..shared.token_policy import TokenPolicy
from ..shared.token_condition_cidr_list import TokenConditionCIDRList

__all__ = ["TokenCreateResponse", "Condition", "ConditionRequestIP"]


class ConditionRequestIP(BaseModel):
    """Client IP restrictions."""

    in_: Optional[List[TokenConditionCIDRList]] = FieldInfo(alias="in", default=None)
    """List of IPv4/IPv6 CIDR addresses."""

    not_in: Optional[List[TokenConditionCIDRList]] = None
    """List of IPv4/IPv6 CIDR addresses."""


class Condition(BaseModel):
    request_ip: Optional[ConditionRequestIP] = None
    """Client IP restrictions."""


class TokenCreateResponse(BaseModel):
    id: Optional[str] = None
    """Token identifier tag."""

    condition: Optional[Condition] = None

    expires_on: Optional[datetime] = None
    """
    The expiration time on or after which the JWT MUST NOT be accepted for
    processing.
    """

    issued_on: Optional[datetime] = None
    """The time on which the token was created."""

    last_used_on: Optional[datetime] = None
    """Last time the token was used."""

    modified_on: Optional[datetime] = None
    """Last time the token was modified."""

    name: Optional[str] = None
    """Token name."""

    not_before: Optional[datetime] = None
    """The time before which the token MUST NOT be accepted for processing."""

    policies: Optional[List[TokenPolicy]] = None
    """List of access policies assigned to the token."""

    status: Optional[Literal["active", "disabled", "expired"]] = None
    """Status of the token."""

    value: Optional[TokenValue] = None
    """The token value."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["KeyGetResponse"]


class KeyGetResponse(BaseModel):
    days_until_next_rotation: Optional[float] = None
    """The number of days until the next key rotation."""

    key_rotation_interval_days: Optional[float] = None
    """The number of days between key rotations."""

    last_key_rotation_at: Optional[datetime] = None
    """The timestamp of the previous key rotation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["TracerouteTestNetworkPathParams"]


class TracerouteTestNetworkPathParams(TypedDict, total=False):
    account_id: Required[str]

    device_id: Required[Annotated[str, PropertyInfo(alias="deviceId")]]
    """Device to filter tracroute result runs to"""

    from_: Required[Annotated[str, PropertyInfo(alias="from")]]
    """Start time for aggregate metrics in ISO ms"""

    interval: Required[Literal["minute", "hour"]]
    """Time interval for aggregate time slots."""

    to: Required[str]
    """End time for aggregate metrics in ISO ms"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["AssetGetResponse"]


class AssetGetResponse(BaseModel):
    description: Optional[str] = None
    """A short description of the custom asset."""

    last_updated: Optional[datetime] = None

    name: Optional[str] = None
    """The unique name of the custom asset.

    Can only contain letters (A-Z, a-z), numbers (0-9), and underscores (\\__).
    """

    size_bytes: Optional[int] = None
    """The size of the asset content in bytes."""

    url: Optional[str] = None
    """The URL where the asset content is fetched from."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, Optional, cast
from datetime import datetime

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.firewall import lockdown_list_params, lockdown_create_params, lockdown_update_params
from ...types.firewall.lockdown import Lockdown
from ...types.firewall.waf.override_url import OverrideURL
from ...types.firewall.configuration_param import ConfigurationParam
from ...types.firewall.lockdown_delete_response import LockdownDeleteResponse

__all__ = ["LockdownsResource", "AsyncLockdownsResource"]


class LockdownsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LockdownsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LockdownsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LockdownsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LockdownsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        configurations: ConfigurationParam,
        urls: SequenceNotStr[OverrideURL],
        description: str | Omit = omit,
        paused: bool | Omit = omit,
        priority: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Lockdown:
        """
        Creates a new Zone Lockdown rule.

        Args:
          zone_id: Defines an identifier.

          configurations: A list of IP addresses or CIDR ranges that will be allowed to access the URLs
              specified in the Zone Lockdown rule. You can include any number of `ip` or
              `ip_range` configurations.

          urls: The URLs to include in the current WAF override. You can use wildcards. Each
              entered URL will be escaped before use, which means you can only use simple
              wildcard patterns.

          description: An informative summary of the rule. This value is sanitized and any tags will be
              removed.

          paused: When true, indicates that the rule is currently paused.

          priority: The priority of the rule to control the processing order. A lower number
              indicates higher priority. If not provided, any rules with a configured priority
              will be processed before rules without a priority.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/firewall/lockdowns", zone_id=zone_id),
            body=maybe_transform(
                {
                    "configurations": configurations,
                    "urls": urls,
                    "description": description,
                    "paused": paused,
                    "priority": priority,
                },
                lockdown_create_params.LockdownCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Lockdown]._unwrapper,
            ),
            cast_to=cast(Type[Lockdown], ResultWrapper[Lockdown]),
        )

    def update(
        self,
        lock_downs_id: str,
        *,
        zone_id: str,
        configurations: ConfigurationParam,
        urls: SequenceNotStr[OverrideURL],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Lockdown:
        """
        Updates an existing Zone Lockdown rule.

        Args:
          zone_id: Defines an identifier.

          lock_downs_id: The unique identifier of the Zone Lockdown rule.

          configurations: A list of IP addresses or CIDR ranges that will be allowed to access the URLs
              specified in the Zone Lockdown rule. You can include any number of `ip` or
              `ip_range` configurations.

          urls: The URLs to include in the current WAF override. You can use wildcards. Each
              entered URL will be escaped before use, which means you can only use simple
              wildcard patterns.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not lock_downs_id:
            raise ValueError(f"Expected a non-empty value for `lock_downs_id` but received {lock_downs_id!r}")
        return self._put(
            path_template(
                "/zones/{zone_id}/firewall/lockdowns/{lock_downs_id}", zone_id=zone_id, lock_downs_id=lock_downs_id
            ),
            body=maybe_transform(
                {
                    "configurations": configurations,
                    "urls": urls,
                },
                lockdown_update_params.LockdownUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Lockdown]._unwrapper,
            ),
            cast_to=cast(Type[Lockdown], ResultWrapper[Lockdown]),
        )

    def list(
        self,
        *,
        zone_id: str,
        created_on: Union[str, datetime] | Omit = omit,
        description: str | Omit = omit,
        description_search: str | Omit = omit,
        ip: str | Omit = omit,
        ip_range_search: str | Omit = omit,
        ip_search: str | Omit = omit,
        modified_on: Union[str, datetime] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        priority: float | Omit = omit,
        uri_search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Lockdown]:
        """Fetches Zone Lockdown rules.

        You can filter the results using several optional
        parameters.

        Args:
          zone_id: Defines an identifier.

          created_on: The timestamp of when the rule was created.

          description: A string to search for in the description of existing rules.

          description_search: A string to search for in the description of existing rules.

          ip: A single IP address to search for in existing rules.

          ip_range_search: A single IP address range to search for in existing rules.

          ip_search: A single IP address to search for in existing rules.

          modified_on: The timestamp of when the rule was last modified.

          page: Page number of paginated results.

          per_page: The maximum number of results per page. You can only set the value to `1` or to
              a multiple of 5 such as `5`, `10`, `15`, or `20`.

          priority: The priority of the rule to control the processing order. A lower number
              indicates higher priority. If not provided, any rules with a configured priority
              will be processed before rules without a priority.

          uri_search: A single URI to search for in the list of URLs of existing rules.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/firewall/lockdowns", zone_id=zone_id),
            page=SyncV4PagePaginationArray[Lockdown],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_on": created_on,
                        "description": description,
                        "description_search": description_search,
                        "ip": ip,
                        "ip_range_search": ip_range_search,
                        "ip_search": ip_search,
                        "modified_on": modified_on,
                        "page": page,
                        "per_page": per_page,
                        "priority": priority,
                        "uri_search": uri_search,
                    },
                    lockdown_list_params.LockdownListParams,
                ),
            ),
            model=Lockdown,
        )

    def delete(
        self,
        lock_downs_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LockdownDeleteResponse]:
        """
        Deletes an existing Zone Lockdown rule.

        Args:
          zone_id: Defines an identifier.

          lock_downs_id: The unique identifier of the Zone Lockdown rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not lock_downs_id:
            raise ValueError(f"Expected a non-empty value for `lock_downs_id` but received {lock_downs_id!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/firewall/lockdowns/{lock_downs_id}", zone_id=zone_id, lock_downs_id=lock_downs_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LockdownDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LockdownDeleteResponse]], ResultWrapper[LockdownDeleteResponse]),
        )

    def get(
        self,
        lock_downs_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Lockdown:
        """
        Fetches the details of a Zone Lockdown rule.

        Args:
          zone_id: Defines an identifier.

          lock_downs_id: The unique identifier of the Zone Lockdown rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not lock_downs_id:
            raise ValueError(f"Expected a non-empty value for `lock_downs_id` but received {lock_downs_id!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/firewall/lockdowns/{lock_downs_id}", zone_id=zone_id, lock_downs_id=lock_downs_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Lockdown]._unwrapper,
            ),
            cast_to=cast(Type[Lockdown], ResultWrapper[Lockdown]),
        )


class AsyncLockdownsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLockdownsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLockdownsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLockdownsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLockdownsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        configurations: ConfigurationParam,
        urls: SequenceNotStr[OverrideURL],
        description: str | Omit = omit,
        paused: bool | Omit = omit,
        priority: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Lockdown:
        """
        Creates a new Zone Lockdown rule.

        Args:
          zone_id: Defines an identifier.

          configurations: A list of IP addresses or CIDR ranges that will be allowed to access the URLs
              specified in the Zone Lockdown rule. You can include any number of `ip` or
              `ip_range` configurations.

          urls: The URLs to include in the current WAF override. You can use wildcards. Each
              entered URL will be escaped before use, which means you can only use simple
              wildcard patterns.

          description: An informative summary of the rule. This value is sanitized and any tags will be
              removed.

          paused: When true, indicates that the rule is currently paused.

          priority: The priority of the rule to control the processing order. A lower number
              indicates higher priority. If not provided, any rules with a configured priority
              will be processed before rules without a priority.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/firewall/lockdowns", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "configurations": configurations,
                    "urls": urls,
                    "description": description,
                    "paused": paused,
                    "priority": priority,
                },
                lockdown_create_params.LockdownCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Lockdown]._unwrapper,
            ),
            cast_to=cast(Type[Lockdown], ResultWrapper[Lockdown]),
        )

    async def update(
        self,
        lock_downs_id: str,
        *,
        zone_id: str,
        configurations: ConfigurationParam,
        urls: SequenceNotStr[OverrideURL],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Lockdown:
        """
        Updates an existing Zone Lockdown rule.

        Args:
          zone_id: Defines an identifier.

          lock_downs_id: The unique identifier of the Zone Lockdown rule.

          configurations: A list of IP addresses or CIDR ranges that will be allowed to access the URLs
              specified in the Zone Lockdown rule. You can include any number of `ip` or
              `ip_range` configurations.

          urls: The URLs to include in the current WAF override. You can use wildcards. Each
              entered URL will be escaped before use, which means you can only use simple
              wildcard patterns.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not lock_downs_id:
            raise ValueError(f"Expected a non-empty value for `lock_downs_id` but received {lock_downs_id!r}")
        return await self._put(
            path_template(
                "/zones/{zone_id}/firewall/lockdowns/{lock_downs_id}", zone_id=zone_id, lock_downs_id=lock_downs_id
            ),
            body=await async_maybe_transform(
                {
                    "configurations": configurations,
                    "urls": urls,
                },
                lockdown_update_params.LockdownUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Lockdown]._unwrapper,
            ),
            cast_to=cast(Type[Lockdown], ResultWrapper[Lockdown]),
        )

    def list(
        self,
        *,
        zone_id: str,
        created_on: Union[str, datetime] | Omit = omit,
        description: str | Omit = omit,
        description_search: str | Omit = omit,
        ip: str | Omit = omit,
        ip_range_search: str | Omit = omit,
        ip_search: str | Omit = omit,
        modified_on: Union[str, datetime] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        priority: float | Omit = omit,
        uri_search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Lockdown, AsyncV4PagePaginationArray[Lockdown]]:
        """Fetches Zone Lockdown rules.

        You can filter the results using several optional
        parameters.

        Args:
          zone_id: Defines an identifier.

          created_on: The timestamp of when the rule was created.

          description: A string to search for in the description of existing rules.

          description_search: A string to search for in the description of existing rules.

          ip: A single IP address to search for in existing rules.

          ip_range_search: A single IP address range to search for in existing rules.

          ip_search: A single IP address to search for in existing rules.

          modified_on: The timestamp of when the rule was last modified.

          page: Page number of paginated results.

          per_page: The maximum number of results per page. You can only set the value to `1` or to
              a multiple of 5 such as `5`, `10`, `15`, or `20`.

          priority: The priority of the rule to control the processing order. A lower number
              indicates higher priority. If not provided, any rules with a configured priority
              will be processed before rules without a priority.

          uri_search: A single URI to search for in the list of URLs of existing rules.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/firewall/lockdowns", zone_id=zone_id),
            page=AsyncV4PagePaginationArray[Lockdown],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_on": created_on,
                        "description": description,
                        "description_search": description_search,
                        "ip": ip,
                        "ip_range_search": ip_range_search,
                        "ip_search": ip_search,
                        "modified_on": modified_on,
                        "page": page,
                        "per_page": per_page,
                        "priority": priority,
                        "uri_search": uri_search,
                    },
                    lockdown_list_params.LockdownListParams,
                ),
            ),
            model=Lockdown,
        )

    async def delete(
        self,
        lock_downs_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LockdownDeleteResponse]:
        """
        Deletes an existing Zone Lockdown rule.

        Args:
          zone_id: Defines an identifier.

          lock_downs_id: The unique identifier of the Zone Lockdown rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not lock_downs_id:
            raise ValueError(f"Expected a non-empty value for `lock_downs_id` but received {lock_downs_id!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/firewall/lockdowns/{lock_downs_id}", zone_id=zone_id, lock_downs_id=lock_downs_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LockdownDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LockdownDeleteResponse]], ResultWrapper[LockdownDeleteResponse]),
        )

    async def get(
        self,
        lock_downs_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Lockdown:
        """
        Fetches the details of a Zone Lockdown rule.

        Args:
          zone_id: Defines an identifier.

          lock_downs_id: The unique identifier of the Zone Lockdown rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not lock_downs_id:
            raise ValueError(f"Expected a non-empty value for `lock_downs_id` but received {lock_downs_id!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/firewall/lockdowns/{lock_downs_id}", zone_id=zone_id, lock_downs_id=lock_downs_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Lockdown]._unwrapper,
            ),
            cast_to=cast(Type[Lockdown], ResultWrapper[Lockdown]),
        )


class LockdownsResourceWithRawResponse:
    def __init__(self, lockdowns: LockdownsResource) -> None:
        self._lockdowns = lockdowns

        self.create = to_raw_response_wrapper(
            lockdowns.create,
        )
        self.update = to_raw_response_wrapper(
            lockdowns.update,
        )
        self.list = to_raw_response_wrapper(
            lockdowns.list,
        )
        self.delete = to_raw_response_wrapper(
            lockdowns.delete,
        )
        self.get = to_raw_response_wrapper(
            lockdowns.get,
        )


class AsyncLockdownsResourceWithRawResponse:
    def __init__(self, lockdowns: AsyncLockdownsResource) -> None:
        self._lockdowns = lockdowns

        self.create = async_to_raw_response_wrapper(
            lockdowns.create,
        )
        self.update = async_to_raw_response_wrapper(
            lockdowns.update,
        )
        self.list = async_to_raw_response_wrapper(
            lockdowns.list,
        )
        self.delete = async_to_raw_response_wrapper(
            lockdowns.delete,
        )
        self.get = async_to_raw_response_wrapper(
            lockdowns.get,
        )


class LockdownsResourceWithStreamingResponse:
    def __init__(self, lockdowns: LockdownsResource) -> None:
        self._lockdowns = lockdowns

        self.create = to_streamed_response_wrapper(
            lockdowns.create,
        )
        self.update = to_streamed_response_wrapper(
            lockdowns.update,
        )
        self.list = to_streamed_response_wrapper(
            lockdowns.list,
        )
        self.delete = to_streamed_response_wrapper(
            lockdowns.delete,
        )
        self.get = to_streamed_response_wrapper(
            lockdowns.get,
        )


class AsyncLockdownsResourceWithStreamingResponse:
    def __init__(self, lockdowns: AsyncLockdownsResource) -> None:
        self._lockdowns = lockdowns

        self.create = async_to_streamed_response_wrapper(
            lockdowns.create,
        )
        self.update = async_to_streamed_response_wrapper(
            lockdowns.update,
        )
        self.list = async_to_streamed_response_wrapper(
            lockdowns.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            lockdowns.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            lockdowns.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["ASPATimeseriesResponse", "Meta", "Serie0"]


class Meta(BaseModel):
    data_time: datetime = FieldInfo(alias="dataTime")
    """Timestamp of the underlying data."""

    query_time: datetime = FieldInfo(alias="queryTime")
    """Timestamp when the query was executed."""


class Serie0(BaseModel):
    timestamps: List[datetime]

    values: List[str]


class ASPATimeseriesResponse(BaseModel):
    meta: Meta

    serie_0: Serie0



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .cookies import (
    CookiesResource,
    AsyncCookiesResource,
    CookiesResourceWithRawResponse,
    AsyncCookiesResourceWithRawResponse,
    CookiesResourceWithStreamingResponse,
    AsyncCookiesResourceWithStreamingResponse,
)
from .scripts import (
    ScriptsResource,
    AsyncScriptsResource,
    ScriptsResourceWithRawResponse,
    AsyncScriptsResourceWithRawResponse,
    ScriptsResourceWithStreamingResponse,
    AsyncScriptsResourceWithStreamingResponse,
)
from .policies import (
    PoliciesResource,
    AsyncPoliciesResource,
    PoliciesResourceWithRawResponse,
    AsyncPoliciesResourceWithRawResponse,
    PoliciesResourceWithStreamingResponse,
    AsyncPoliciesResourceWithStreamingResponse,
)
from .connections import (
    ConnectionsResource,
    AsyncConnectionsResource,
    ConnectionsResourceWithRawResponse,
    AsyncConnectionsResourceWithRawResponse,
    ConnectionsResourceWithStreamingResponse,
    AsyncConnectionsResourceWithStreamingResponse,
)
from .page_shield import (
    PageShieldResource,
    AsyncPageShieldResource,
    PageShieldResourceWithRawResponse,
    AsyncPageShieldResourceWithRawResponse,
    PageShieldResourceWithStreamingResponse,
    AsyncPageShieldResourceWithStreamingResponse,
)

__all__ = [
    "PoliciesResource",
    "AsyncPoliciesResource",
    "PoliciesResourceWithRawResponse",
    "AsyncPoliciesResourceWithRawResponse",
    "PoliciesResourceWithStreamingResponse",
    "AsyncPoliciesResourceWithStreamingResponse",
    "ConnectionsResource",
    "AsyncConnectionsResource",
    "ConnectionsResourceWithRawResponse",
    "AsyncConnectionsResourceWithRawResponse",
    "ConnectionsResourceWithStreamingResponse",
    "AsyncConnectionsResourceWithStreamingResponse",
    "ScriptsResource",
    "AsyncScriptsResource",
    "ScriptsResourceWithRawResponse",
    "AsyncScriptsResourceWithRawResponse",
    "ScriptsResourceWithStreamingResponse",
    "AsyncScriptsResourceWithStreamingResponse",
    "CookiesResource",
    "AsyncCookiesResource",
    "CookiesResourceWithRawResponse",
    "AsyncCookiesResourceWithRawResponse",
    "CookiesResourceWithStreamingResponse",
    "AsyncCookiesResourceWithStreamingResponse",
    "PageShieldResource",
    "AsyncPageShieldResource",
    "PageShieldResourceWithRawResponse",
    "AsyncPageShieldResourceWithRawResponse",
    "PageShieldResourceWithStreamingResponse",
    "AsyncPageShieldResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["SentineloneS2sInput"]


class SentineloneS2sInput(BaseModel):
    connection_id: str
    """Posture Integration ID."""

    active_threats: Optional[float] = None
    """The Number of active threats."""

    infected: Optional[bool] = None
    """Whether device is infected."""

    is_active: Optional[bool] = None
    """Whether device is active."""

    network_status: Optional[Literal["connected", "disconnected", "disconnecting", "connecting"]] = None
    """Network status of device."""

    operational_state: Optional[
        Literal[
            "na",
            "partially_disabled",
            "auto_fully_disabled",
            "fully_disabled",
            "auto_partially_disabled",
            "disabled_error",
            "db_corruption",
        ]
    ] = None
    """Agent operational state."""

    operator: Optional[Literal["<", "<=", ">", ">=", "=="]] = None
    """Operator."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Target", "Constraint"]


class Constraint(BaseModel):
    """String constraint."""

    operator: Literal["matches", "contains", "equals", "not_equal", "not_contain"]
    """
    The matches operator can use asterisks and pipes as wildcard and 'or' operators.
    """

    value: str
    """The URL pattern to match against the current request.

    The pattern may contain up to four asterisks ('\\**') as placeholders.
    """


class Target(BaseModel):
    """URL target."""

    constraint: Optional[Constraint] = None
    """String constraint."""

    target: Optional[Literal["url"]] = None
    """A target based on the URL of the request."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from ...._models import BaseModel

__all__ = ["CountryListResponse", "CountryListResponseItem", "CountryListResponseItemResult"]


class CountryListResponseItemResult(BaseModel):
    alpha3: str

    name: str


class CountryListResponseItem(BaseModel):
    result: List[CountryListResponseItemResult]

    success: str


CountryListResponse: TypeAlias = List[CountryListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ASNListParams"]


class ASNListParams(TypedDict, total=False):
    asn: str
    """Filters results by Autonomous System.

    Specify one or more Autonomous System Numbers (ASNs) as a comma-separated list.
    """

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    limit: int
    """Limits the number of objects returned in the response."""

    location: str
    """Filters results by location. Specify an alpha-2 location code."""

    offset: int
    """Skips the specified number of objects before fetching the results."""

    order_by: Annotated[Literal["ASN", "POPULATION"], PropertyInfo(alias="orderBy")]
    """Specifies the metric to order the ASNs by."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import is_given, path_template, maybe_transform, strip_not_given, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.r2.buckets.domains import custom_create_params, custom_update_params
from .....types.r2.buckets.domains.custom_get_response import CustomGetResponse
from .....types.r2.buckets.domains.custom_list_response import CustomListResponse
from .....types.r2.buckets.domains.custom_create_response import CustomCreateResponse
from .....types.r2.buckets.domains.custom_delete_response import CustomDeleteResponse
from .....types.r2.buckets.domains.custom_update_response import CustomUpdateResponse

__all__ = ["CustomResource", "AsyncCustomResource"]


class CustomResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CustomResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CustomResourceWithStreamingResponse(self)

    def create(
        self,
        bucket_name: str,
        *,
        account_id: str,
        domain: str,
        enabled: bool,
        zone_id: str,
        ciphers: SequenceNotStr[str] | Omit = omit,
        min_tls: Literal["1.0", "1.1", "1.2", "1.3"] | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomCreateResponse:
        """
        Register a new custom domain for an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          domain: Name of the custom domain to be added.

          enabled: Whether to enable public bucket access at the custom domain. If undefined, the
              domain will be enabled.

          zone_id: Zone ID of the custom domain.

          ciphers: An allowlist of ciphers for TLS termination. These ciphers must be in the
              BoringSSL format.

          min_tls: Minimum TLS Version the custom domain will accept for incoming connections. If
              not set, defaults to 1.0.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._post(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/domains/custom",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            body=maybe_transform(
                {
                    "domain": domain,
                    "enabled": enabled,
                    "zone_id": zone_id,
                    "ciphers": ciphers,
                    "min_tls": min_tls,
                },
                custom_create_params.CustomCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomCreateResponse], ResultWrapper[CustomCreateResponse]),
        )

    def update(
        self,
        domain: str,
        *,
        account_id: str,
        bucket_name: str,
        ciphers: SequenceNotStr[str] | Omit = omit,
        enabled: bool | Omit = omit,
        min_tls: Literal["1.0", "1.1", "1.2", "1.3"] | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomUpdateResponse:
        """
        Edit the configuration for a custom domain on an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          domain: Name of the custom domain.

          ciphers: An allowlist of ciphers for TLS termination. These ciphers must be in the
              BoringSSL format.

          enabled: Whether to enable public bucket access at the specified custom domain.

          min_tls: Minimum TLS Version the custom domain will accept for incoming connections. If
              not set, defaults to previous value.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not domain:
            raise ValueError(f"Expected a non-empty value for `domain` but received {domain!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._put(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/domains/custom/{domain}",
                account_id=account_id,
                bucket_name=bucket_name,
                domain=domain,
            ),
            body=maybe_transform(
                {
                    "ciphers": ciphers,
                    "enabled": enabled,
                    "min_tls": min_tls,
                },
                custom_update_params.CustomUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomUpdateResponse], ResultWrapper[CustomUpdateResponse]),
        )

    def list(
        self,
        bucket_name: str,
        *,
        account_id: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomListResponse:
        """
        Gets a list of all custom domains registered with an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._get(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/domains/custom",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomListResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomListResponse], ResultWrapper[CustomListResponse]),
        )

    def delete(
        self,
        domain: str,
        *,
        account_id: str,
        bucket_name: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomDeleteResponse:
        """
        Remove custom domain registration from an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          domain: Name of the custom domain.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not domain:
            raise ValueError(f"Expected a non-empty value for `domain` but received {domain!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._delete(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/domains/custom/{domain}",
                account_id=account_id,
                bucket_name=bucket_name,
                domain=domain,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomDeleteResponse], ResultWrapper[CustomDeleteResponse]),
        )

    def get(
        self,
        domain: str,
        *,
        account_id: str,
        bucket_name: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomGetResponse:
        """
        Get the configuration for a custom domain on an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          domain: Name of the custom domain.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not domain:
            raise ValueError(f"Expected a non-empty value for `domain` but received {domain!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._get(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/domains/custom/{domain}",
                account_id=account_id,
                bucket_name=bucket_name,
                domain=domain,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomGetResponse], ResultWrapper[CustomGetResponse]),
        )


class AsyncCustomResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCustomResourceWithStreamingResponse(self)

    async def create(
        self,
        bucket_name: str,
        *,
        account_id: str,
        domain: str,
        enabled: bool,
        zone_id: str,
        ciphers: SequenceNotStr[str] | Omit = omit,
        min_tls: Literal["1.0", "1.1", "1.2", "1.3"] | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomCreateResponse:
        """
        Register a new custom domain for an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          domain: Name of the custom domain to be added.

          enabled: Whether to enable public bucket access at the custom domain. If undefined, the
              domain will be enabled.

          zone_id: Zone ID of the custom domain.

          ciphers: An allowlist of ciphers for TLS termination. These ciphers must be in the
              BoringSSL format.

          min_tls: Minimum TLS Version the custom domain will accept for incoming connections. If
              not set, defaults to 1.0.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._post(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/domains/custom",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            body=await async_maybe_transform(
                {
                    "domain": domain,
                    "enabled": enabled,
                    "zone_id": zone_id,
                    "ciphers": ciphers,
                    "min_tls": min_tls,
                },
                custom_create_params.CustomCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomCreateResponse], ResultWrapper[CustomCreateResponse]),
        )

    async def update(
        self,
        domain: str,
        *,
        account_id: str,
        bucket_name: str,
        ciphers: SequenceNotStr[str] | Omit = omit,
        enabled: bool | Omit = omit,
        min_tls: Literal["1.0", "1.1", "1.2", "1.3"] | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomUpdateResponse:
        """
        Edit the configuration for a custom domain on an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          domain: Name of the custom domain.

          ciphers: An allowlist of ciphers for TLS termination. These ciphers must be in the
              BoringSSL format.

          enabled: Whether to enable public bucket access at the specified custom domain.

          min_tls: Minimum TLS Version the custom domain will accept for incoming connections. If
              not set, defaults to previous value.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not domain:
            raise ValueError(f"Expected a non-empty value for `domain` but received {domain!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._put(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/domains/custom/{domain}",
                account_id=account_id,
                bucket_name=bucket_name,
                domain=domain,
            ),
            body=await async_maybe_transform(
                {
                    "ciphers": ciphers,
                    "enabled": enabled,
                    "min_tls": min_tls,
                },
                custom_update_params.CustomUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomUpdateResponse], ResultWrapper[CustomUpdateResponse]),
        )

    async def list(
        self,
        bucket_name: str,
        *,
        account_id: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomListResponse:
        """
        Gets a list of all custom domains registered with an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._get(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/domains/custom",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomListResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomListResponse], ResultWrapper[CustomListResponse]),
        )

    async def delete(
        self,
        domain: str,
        *,
        account_id: str,
        bucket_name: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomDeleteResponse:
        """
        Remove custom domain registration from an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          domain: Name of the custom domain.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not domain:
            raise ValueError(f"Expected a non-empty value for `domain` but received {domain!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._delete(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/domains/custom/{domain}",
                account_id=account_id,
                bucket_name=bucket_name,
                domain=domain,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomDeleteResponse], ResultWrapper[CustomDeleteResponse]),
        )

    async def get(
        self,
        domain: str,
        *,
        account_id: str,
        bucket_name: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomGetResponse:
        """
        Get the configuration for a custom domain on an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          domain: Name of the custom domain.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not domain:
            raise ValueError(f"Expected a non-empty value for `domain` but received {domain!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._get(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}/domains/custom/{domain}",
                account_id=account_id,
                bucket_name=bucket_name,
                domain=domain,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomGetResponse], ResultWrapper[CustomGetResponse]),
        )


class CustomResourceWithRawResponse:
    def __init__(self, custom: CustomResource) -> None:
        self._custom = custom

        self.create = to_raw_response_wrapper(
            custom.create,
        )
        self.update = to_raw_response_wrapper(
            custom.update,
        )
        self.list = to_raw_response_wrapper(
            custom.list,
        )
        self.delete = to_raw_response_wrapper(
            custom.delete,
        )
        self.get = to_raw_response_wrapper(
            custom.get,
        )


class AsyncCustomResourceWithRawResponse:
    def __init__(self, custom: AsyncCustomResource) -> None:
        self._custom = custom

        self.create = async_to_raw_response_wrapper(
            custom.create,
        )
        self.update = async_to_raw_response_wrapper(
            custom.update,
        )
        self.list = async_to_raw_response_wrapper(
            custom.list,
        )
        self.delete = async_to_raw_response_wrapper(
            custom.delete,
        )
        self.get = async_to_raw_response_wrapper(
            custom.get,
        )


class CustomResourceWithStreamingResponse:
    def __init__(self, custom: CustomResource) -> None:
        self._custom = custom

        self.create = to_streamed_response_wrapper(
            custom.create,
        )
        self.update = to_streamed_response_wrapper(
            custom.update,
        )
        self.list = to_streamed_response_wrapper(
            custom.list,
        )
        self.delete = to_streamed_response_wrapper(
            custom.delete,
        )
        self.get = to_streamed_response_wrapper(
            custom.get,
        )


class AsyncCustomResourceWithStreamingResponse:
    def __init__(self, custom: AsyncCustomResource) -> None:
        self._custom = custom

        self.create = async_to_streamed_response_wrapper(
            custom.create,
        )
        self.update = async_to_streamed_response_wrapper(
            custom.update,
        )
        self.list = async_to_streamed_response_wrapper(
            custom.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            custom.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            custom.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["BGPPrefixEditParams", "OnDemand"]


class BGPPrefixEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier of a Cloudflare account."""

    prefix_id: Required[str]
    """Identifier of an IP Prefix."""

    asn_prepend_count: int
    """Number of times to prepend the Cloudflare ASN to the BGP AS-Path attribute"""

    auto_advertise_withdraw: bool
    """
    Determines if Cloudflare advertises a BYOIP BGP prefix even when there is no
    matching BGP prefix in the Magic routing table. When true, Cloudflare will
    automatically withdraw the BGP prefix when there are no matching BGP routes, and
    will resume advertising when there is at least one matching BGP route.
    """

    on_demand: OnDemand


class OnDemand(TypedDict, total=False):
    advertised: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SessionGetSessionsParams"]


class SessionGetSessionsParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    associated_id: str
    """ID of the meeting that sessions should be associated with"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The end time range for which you want to retrieve the meetings.

    The time must be specified in ISO format.
    """

    page_no: float
    """The page number from which you want your page search results to be displayed."""

    participants: str

    per_page: float
    """Number of results per page"""

    search: str
    """
    Search string that matches sessions based on meeting title, meeting ID, and
    session ID
    """

    sort_by: Literal["minutesConsumed", "createdAt"]

    sort_order: Literal["ASC", "DESC"]

    start_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The start time range for which you want to retrieve the meetings.

    The time must be specified in ISO format.
    """

    status: Literal["LIVE", "ENDED"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["RegistrarSearchParams"]


class RegistrarSearchParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    q: Required[str]
    """The search term to find domain suggestions.

    Accepts keywords, phrases, or full domain names.

    - Phrases: "coffee shop" returns coffeeshop.com, mycoffeeshop.net, etc.
    - Domain names: "example.com" returns example.com and variations across
      extensions
    """

    extensions: SequenceNotStr[str]
    """Limits results to specific domain extensions from the supported set.

    If not specified, returns results across all supported extensions. Extensions
    not in the supported set are silently ignored.
    """

    limit: int
    """Maximum number of domain suggestions to return.

    Defaults to 20 if not specified.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = [
    "DetectionGetResponse",
    "Attachment",
    "Finding",
    "Header",
    "Link",
    "SenderInfo",
    "ThreatCategory",
    "Validation",
]


class Attachment(BaseModel):
    size: int
    """Size of the attachment in bytes"""

    content_type: Optional[str] = None
    """MIME type of the attachment"""

    detection: Optional[
        Literal[
            "MALICIOUS",
            "MALICIOUS-BEC",
            "SUSPICIOUS",
            "SPOOF",
            "SPAM",
            "BULK",
            "ENCRYPTED",
            "EXTERNAL",
            "UNKNOWN",
            "NONE",
        ]
    ] = None
    """Detection result for this attachment"""

    encrypted: Optional[bool] = None
    """Whether the attachment is encrypted"""

    filename: Optional[str] = None
    """Name of the attached file"""

    md5: Optional[str] = None
    """MD5 hash of the attachment"""

    name: Optional[str] = None
    """Attachment name (alternative to filename)"""

    sha1: Optional[str] = None
    """SHA1 hash of the attachment"""

    sha256: Optional[str] = None
    """SHA256 hash of the attachment"""


class Finding(BaseModel):
    attachment: Optional[str] = None

    detail: Optional[str] = None

    detection: Optional[
        Literal[
            "MALICIOUS",
            "MALICIOUS-BEC",
            "SUSPICIOUS",
            "SPOOF",
            "SPAM",
            "BULK",
            "ENCRYPTED",
            "EXTERNAL",
            "UNKNOWN",
            "NONE",
        ]
    ] = None

    field: Optional[str] = None

    name: Optional[str] = None

    portion: Optional[str] = None

    reason: Optional[str] = None

    score: Optional[float] = None

    value: Optional[str] = None


class Header(BaseModel):
    name: str

    value: str


class Link(BaseModel):
    href: str

    text: Optional[str] = None


class SenderInfo(BaseModel):
    as_name: Optional[str] = None
    """The name of the autonomous system."""

    as_number: Optional[int] = None
    """The number of the autonomous system."""

    geo: Optional[str] = None

    ip: Optional[str] = None

    pld: Optional[str] = None


class ThreatCategory(BaseModel):
    id: Optional[int] = None

    description: Optional[str] = None

    name: Optional[str] = None


class Validation(BaseModel):
    comment: Optional[str] = None

    dkim: Optional[Literal["pass", "neutral", "fail", "error", "none"]] = None

    dmarc: Optional[Literal["pass", "neutral", "fail", "error", "none"]] = None

    spf: Optional[Literal["pass", "neutral", "fail", "error", "none"]] = None


class DetectionGetResponse(BaseModel):
    action: str

    attachments: List[Attachment]

    findings: Optional[List[Finding]] = None

    headers: List[Header]

    links: List[Link]

    sender_info: SenderInfo

    threat_categories: List[ThreatCategory]

    validation: Validation

    final_disposition: Optional[
        Literal[
            "MALICIOUS",
            "MALICIOUS-BEC",
            "SUSPICIOUS",
            "SPOOF",
            "SPAM",
            "BULK",
            "ENCRYPTED",
            "EXTERNAL",
            "UNKNOWN",
            "NONE",
        ]
    ] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["RewriteAction"]


class RewriteAction(BaseModel):
    """
    Specifies that, when a WAF rule matches, its configured action will be replaced by the action configured in this object.
    """

    block: Optional[Literal["challenge", "block", "simulate", "disable", "default"]] = None
    """The WAF rule action to apply."""

    challenge: Optional[Literal["challenge", "block", "simulate", "disable", "default"]] = None
    """The WAF rule action to apply."""

    default: Optional[Literal["challenge", "block", "simulate", "disable", "default"]] = None
    """The WAF rule action to apply."""

    disable: Optional[Literal["challenge", "block", "simulate", "disable", "default"]] = None
    """The WAF rule action to apply."""

    simulate: Optional[Literal["challenge", "block", "simulate", "disable", "default"]] = None
    """The WAF rule action to apply."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .membership import Membership as Membership
from .membership_list_params import MembershipListParams as MembershipListParams
from .membership_get_response import MembershipGetResponse as MembershipGetResponse
from .membership_update_params import MembershipUpdateParams as MembershipUpdateParams
from .membership_delete_response import MembershipDeleteResponse as MembershipDeleteResponse
from .membership_update_response import MembershipUpdateResponse as MembershipUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .configuration import (
    ConfigurationResource,
    AsyncConfigurationResource,
    ConfigurationResourceWithRawResponse,
    AsyncConfigurationResourceWithRawResponse,
    ConfigurationResourceWithStreamingResponse,
    AsyncConfigurationResourceWithStreamingResponse,
)
from .token_validation import (
    TokenValidationResource,
    AsyncTokenValidationResource,
    TokenValidationResourceWithRawResponse,
    AsyncTokenValidationResourceWithRawResponse,
    TokenValidationResourceWithStreamingResponse,
    AsyncTokenValidationResourceWithStreamingResponse,
)

__all__ = [
    "ConfigurationResource",
    "AsyncConfigurationResource",
    "ConfigurationResourceWithRawResponse",
    "AsyncConfigurationResourceWithRawResponse",
    "ConfigurationResourceWithStreamingResponse",
    "AsyncConfigurationResourceWithStreamingResponse",
    "RulesResource",
    "AsyncRulesResource",
    "RulesResourceWithRawResponse",
    "AsyncRulesResourceWithRawResponse",
    "RulesResourceWithStreamingResponse",
    "AsyncRulesResourceWithStreamingResponse",
    "TokenValidationResource",
    "AsyncTokenValidationResource",
    "TokenValidationResourceWithRawResponse",
    "AsyncTokenValidationResourceWithRawResponse",
    "TokenValidationResourceWithStreamingResponse",
    "AsyncTokenValidationResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["CertificateGetResponse"]


class CertificateGetResponse(BaseModel):
    id: Optional[str] = None
    """Identify the certificate with a UUID."""

    binding_status: Optional[Literal["pending_deployment", "available", "pending_deletion", "inactive"]] = None
    """Indicate the read-only deployment status of the certificate on Cloudflare's
    edge.

    Gateway TLS interception can use certificates in the 'available' (previously
    called 'active') state.
    """

    certificate: Optional[str] = None
    """Provide the CA certificate (read-only)."""

    created_at: Optional[datetime] = None

    expires_on: Optional[datetime] = None

    fingerprint: Optional[str] = None
    """Provide the SHA256 fingerprint of the certificate (read-only)."""

    in_use: Optional[bool] = None
    """Indicate whether Gateway TLS interception uses this certificate (read-only).

    You cannot set this value directly. To configure interception, use the Gateway
    configuration setting named `certificate` (read-only).
    """

    issuer_org: Optional[str] = None
    """Indicate the organization that issued the certificate (read-only)."""

    issuer_raw: Optional[str] = None
    """Provide the entire issuer field of the certificate (read-only)."""

    type: Optional[Literal["custom", "gateway_managed"]] = None
    """Indicate the read-only certificate type, BYO-PKI (custom) or Gateway-managed."""

    updated_at: Optional[datetime] = None

    uploaded_on: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .edge import (
    EdgeResource,
    AsyncEdgeResource,
    EdgeResourceWithRawResponse,
    AsyncEdgeResourceWithRawResponse,
    EdgeResourceWithStreamingResponse,
    AsyncEdgeResourceWithStreamingResponse,
)
from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .logpush import (
    LogpushResource,
    AsyncLogpushResource,
    LogpushResourceWithRawResponse,
    AsyncLogpushResourceWithRawResponse,
    LogpushResourceWithStreamingResponse,
    AsyncLogpushResourceWithStreamingResponse,
)
from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from .validate import (
    ValidateResource,
    AsyncValidateResource,
    ValidateResourceWithRawResponse,
    AsyncValidateResourceWithRawResponse,
    ValidateResourceWithStreamingResponse,
    AsyncValidateResourceWithStreamingResponse,
)
from .ownership import (
    OwnershipResource,
    AsyncOwnershipResource,
    OwnershipResourceWithRawResponse,
    AsyncOwnershipResourceWithRawResponse,
    OwnershipResourceWithStreamingResponse,
    AsyncOwnershipResourceWithStreamingResponse,
)

__all__ = [
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
    "EdgeResource",
    "AsyncEdgeResource",
    "EdgeResourceWithRawResponse",
    "AsyncEdgeResourceWithRawResponse",
    "EdgeResourceWithStreamingResponse",
    "AsyncEdgeResourceWithStreamingResponse",
    "JobsResource",
    "AsyncJobsResource",
    "JobsResourceWithRawResponse",
    "AsyncJobsResourceWithRawResponse",
    "JobsResourceWithStreamingResponse",
    "AsyncJobsResourceWithStreamingResponse",
    "OwnershipResource",
    "AsyncOwnershipResource",
    "OwnershipResourceWithRawResponse",
    "AsyncOwnershipResourceWithRawResponse",
    "OwnershipResourceWithStreamingResponse",
    "AsyncOwnershipResourceWithStreamingResponse",
    "ValidateResource",
    "AsyncValidateResource",
    "ValidateResourceWithRawResponse",
    "AsyncValidateResourceWithRawResponse",
    "ValidateResourceWithStreamingResponse",
    "AsyncValidateResourceWithStreamingResponse",
    "LogpushResource",
    "AsyncLogpushResource",
    "LogpushResourceWithRawResponse",
    "AsyncLogpushResourceWithRawResponse",
    "LogpushResourceWithStreamingResponse",
    "AsyncLogpushResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from .reverse_dns import (
    ReverseDNSResource,
    AsyncReverseDNSResource,
    ReverseDNSResourceWithRawResponse,
    AsyncReverseDNSResourceWithRawResponse,
    ReverseDNSResourceWithStreamingResponse,
    AsyncReverseDNSResourceWithStreamingResponse,
)
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from .analytics.analytics import (
    AnalyticsResource,
    AsyncAnalyticsResource,
    AnalyticsResourceWithRawResponse,
    AsyncAnalyticsResourceWithRawResponse,
    AnalyticsResourceWithStreamingResponse,
    AsyncAnalyticsResourceWithStreamingResponse,
)
from ...types.dns_firewall import (
    dns_firewall_edit_params,
    dns_firewall_list_params,
    dns_firewall_create_params,
)
from ...types.dns_firewall.upstream_ips import UpstreamIPs
from ...types.dns_firewall.attack_mitigation_param import AttackMitigationParam
from ...types.dns_firewall.dns_firewall_get_response import DNSFirewallGetResponse
from ...types.dns_firewall.dns_firewall_edit_response import DNSFirewallEditResponse
from ...types.dns_firewall.dns_firewall_list_response import DNSFirewallListResponse
from ...types.dns_firewall.dns_firewall_create_response import DNSFirewallCreateResponse
from ...types.dns_firewall.dns_firewall_delete_response import DNSFirewallDeleteResponse

__all__ = ["DNSFirewallResource", "AsyncDNSFirewallResource"]


class DNSFirewallResource(SyncAPIResource):
    @cached_property
    def analytics(self) -> AnalyticsResource:
        return AnalyticsResource(self._client)

    @cached_property
    def reverse_dns(self) -> ReverseDNSResource:
        return ReverseDNSResource(self._client)

    @cached_property
    def with_raw_response(self) -> DNSFirewallResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DNSFirewallResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DNSFirewallResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DNSFirewallResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        upstream_ips: SequenceNotStr[UpstreamIPs],
        attack_mitigation: Optional[AttackMitigationParam] | Omit = omit,
        deprecate_any_requests: bool | Omit = omit,
        ecs_fallback: bool | Omit = omit,
        maximum_cache_ttl: float | Omit = omit,
        minimum_cache_ttl: float | Omit = omit,
        negative_cache_ttl: Optional[float] | Omit = omit,
        ratelimit: Optional[float] | Omit = omit,
        retries: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSFirewallCreateResponse]:
        """
        Create a DNS Firewall cluster

        Args:
          account_id: Identifier.

          name: DNS Firewall cluster name

          attack_mitigation: Attack mitigation settings

          deprecate_any_requests: Whether to refuse to answer queries for the ANY type

          ecs_fallback: Whether to forward client IP (resolver) subnet if no EDNS Client Subnet is sent

          maximum_cache_ttl: By default, Cloudflare attempts to cache responses for as long as indicated by
              the TTL received from upstream nameservers. This setting sets an upper bound on
              this duration. For caching purposes, higher TTLs will be decreased to the
              maximum value defined by this setting.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

          minimum_cache_ttl: By default, Cloudflare attempts to cache responses for as long as indicated by
              the TTL received from upstream nameservers. This setting sets a lower bound on
              this duration. For caching purposes, lower TTLs will be increased to the minimum
              value defined by this setting.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

              Note that, even with this setting, there is no guarantee that a response will be
              cached for at least the specified duration. Cached responses may be removed
              earlier for capacity or other operational reasons.

          negative_cache_ttl: This setting controls how long DNS Firewall should cache negative responses
              (e.g., NXDOMAIN) from the upstream servers.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

          ratelimit: Ratelimit in queries per second per datacenter (applies to DNS queries sent to
              the upstream nameservers configured on the cluster)

          retries: Number of retries for fetching DNS responses from upstream nameservers (not
              counting the initial attempt)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/dns_firewall", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "upstream_ips": upstream_ips,
                    "attack_mitigation": attack_mitigation,
                    "deprecate_any_requests": deprecate_any_requests,
                    "ecs_fallback": ecs_fallback,
                    "maximum_cache_ttl": maximum_cache_ttl,
                    "minimum_cache_ttl": minimum_cache_ttl,
                    "negative_cache_ttl": negative_cache_ttl,
                    "ratelimit": ratelimit,
                    "retries": retries,
                },
                dns_firewall_create_params.DNSFirewallCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSFirewallCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSFirewallCreateResponse]], ResultWrapper[DNSFirewallCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[DNSFirewallListResponse]:
        """
        List DNS Firewall clusters for an account

        Args:
          account_id: Identifier.

          page: Page number of paginated results

          per_page: Number of clusters per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dns_firewall", account_id=account_id),
            page=SyncV4PagePaginationArray[DNSFirewallListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    dns_firewall_list_params.DNSFirewallListParams,
                ),
            ),
            model=DNSFirewallListResponse,
        )

    def delete(
        self,
        dns_firewall_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSFirewallDeleteResponse]:
        """
        Delete a DNS Firewall cluster

        Args:
          account_id: Identifier.

          dns_firewall_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dns_firewall_id:
            raise ValueError(f"Expected a non-empty value for `dns_firewall_id` but received {dns_firewall_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/dns_firewall/{dns_firewall_id}",
                account_id=account_id,
                dns_firewall_id=dns_firewall_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSFirewallDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSFirewallDeleteResponse]], ResultWrapper[DNSFirewallDeleteResponse]),
        )

    def edit(
        self,
        dns_firewall_id: str,
        *,
        account_id: str,
        attack_mitigation: Optional[AttackMitigationParam] | Omit = omit,
        deprecate_any_requests: bool | Omit = omit,
        ecs_fallback: bool | Omit = omit,
        maximum_cache_ttl: float | Omit = omit,
        minimum_cache_ttl: float | Omit = omit,
        name: str | Omit = omit,
        negative_cache_ttl: Optional[float] | Omit = omit,
        ratelimit: Optional[float] | Omit = omit,
        retries: float | Omit = omit,
        upstream_ips: SequenceNotStr[UpstreamIPs] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSFirewallEditResponse]:
        """
        Modify the configuration of a DNS Firewall cluster

        Args:
          account_id: Identifier.

          dns_firewall_id: Identifier.

          attack_mitigation: Attack mitigation settings

          deprecate_any_requests: Whether to refuse to answer queries for the ANY type

          ecs_fallback: Whether to forward client IP (resolver) subnet if no EDNS Client Subnet is sent

          maximum_cache_ttl: By default, Cloudflare attempts to cache responses for as long as indicated by
              the TTL received from upstream nameservers. This setting sets an upper bound on
              this duration. For caching purposes, higher TTLs will be decreased to the
              maximum value defined by this setting.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

          minimum_cache_ttl: By default, Cloudflare attempts to cache responses for as long as indicated by
              the TTL received from upstream nameservers. This setting sets a lower bound on
              this duration. For caching purposes, lower TTLs will be increased to the minimum
              value defined by this setting.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

              Note that, even with this setting, there is no guarantee that a response will be
              cached for at least the specified duration. Cached responses may be removed
              earlier for capacity or other operational reasons.

          name: DNS Firewall cluster name

          negative_cache_ttl: This setting controls how long DNS Firewall should cache negative responses
              (e.g., NXDOMAIN) from the upstream servers.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

          ratelimit: Ratelimit in queries per second per datacenter (applies to DNS queries sent to
              the upstream nameservers configured on the cluster)

          retries: Number of retries for fetching DNS responses from upstream nameservers (not
              counting the initial attempt)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dns_firewall_id:
            raise ValueError(f"Expected a non-empty value for `dns_firewall_id` but received {dns_firewall_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/dns_firewall/{dns_firewall_id}",
                account_id=account_id,
                dns_firewall_id=dns_firewall_id,
            ),
            body=maybe_transform(
                {
                    "attack_mitigation": attack_mitigation,
                    "deprecate_any_requests": deprecate_any_requests,
                    "ecs_fallback": ecs_fallback,
                    "maximum_cache_ttl": maximum_cache_ttl,
                    "minimum_cache_ttl": minimum_cache_ttl,
                    "name": name,
                    "negative_cache_ttl": negative_cache_ttl,
                    "ratelimit": ratelimit,
                    "retries": retries,
                    "upstream_ips": upstream_ips,
                },
                dns_firewall_edit_params.DNSFirewallEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSFirewallEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSFirewallEditResponse]], ResultWrapper[DNSFirewallEditResponse]),
        )

    def get(
        self,
        dns_firewall_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSFirewallGetResponse]:
        """
        Show a single DNS Firewall cluster for an account

        Args:
          account_id: Identifier.

          dns_firewall_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dns_firewall_id:
            raise ValueError(f"Expected a non-empty value for `dns_firewall_id` but received {dns_firewall_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/dns_firewall/{dns_firewall_id}",
                account_id=account_id,
                dns_firewall_id=dns_firewall_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSFirewallGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSFirewallGetResponse]], ResultWrapper[DNSFirewallGetResponse]),
        )


class AsyncDNSFirewallResource(AsyncAPIResource):
    @cached_property
    def analytics(self) -> AsyncAnalyticsResource:
        return AsyncAnalyticsResource(self._client)

    @cached_property
    def reverse_dns(self) -> AsyncReverseDNSResource:
        return AsyncReverseDNSResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDNSFirewallResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDNSFirewallResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDNSFirewallResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDNSFirewallResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        upstream_ips: SequenceNotStr[UpstreamIPs],
        attack_mitigation: Optional[AttackMitigationParam] | Omit = omit,
        deprecate_any_requests: bool | Omit = omit,
        ecs_fallback: bool | Omit = omit,
        maximum_cache_ttl: float | Omit = omit,
        minimum_cache_ttl: float | Omit = omit,
        negative_cache_ttl: Optional[float] | Omit = omit,
        ratelimit: Optional[float] | Omit = omit,
        retries: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSFirewallCreateResponse]:
        """
        Create a DNS Firewall cluster

        Args:
          account_id: Identifier.

          name: DNS Firewall cluster name

          attack_mitigation: Attack mitigation settings

          deprecate_any_requests: Whether to refuse to answer queries for the ANY type

          ecs_fallback: Whether to forward client IP (resolver) subnet if no EDNS Client Subnet is sent

          maximum_cache_ttl: By default, Cloudflare attempts to cache responses for as long as indicated by
              the TTL received from upstream nameservers. This setting sets an upper bound on
              this duration. For caching purposes, higher TTLs will be decreased to the
              maximum value defined by this setting.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

          minimum_cache_ttl: By default, Cloudflare attempts to cache responses for as long as indicated by
              the TTL received from upstream nameservers. This setting sets a lower bound on
              this duration. For caching purposes, lower TTLs will be increased to the minimum
              value defined by this setting.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

              Note that, even with this setting, there is no guarantee that a response will be
              cached for at least the specified duration. Cached responses may be removed
              earlier for capacity or other operational reasons.

          negative_cache_ttl: This setting controls how long DNS Firewall should cache negative responses
              (e.g., NXDOMAIN) from the upstream servers.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

          ratelimit: Ratelimit in queries per second per datacenter (applies to DNS queries sent to
              the upstream nameservers configured on the cluster)

          retries: Number of retries for fetching DNS responses from upstream nameservers (not
              counting the initial attempt)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/dns_firewall", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "upstream_ips": upstream_ips,
                    "attack_mitigation": attack_mitigation,
                    "deprecate_any_requests": deprecate_any_requests,
                    "ecs_fallback": ecs_fallback,
                    "maximum_cache_ttl": maximum_cache_ttl,
                    "minimum_cache_ttl": minimum_cache_ttl,
                    "negative_cache_ttl": negative_cache_ttl,
                    "ratelimit": ratelimit,
                    "retries": retries,
                },
                dns_firewall_create_params.DNSFirewallCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSFirewallCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSFirewallCreateResponse]], ResultWrapper[DNSFirewallCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DNSFirewallListResponse, AsyncV4PagePaginationArray[DNSFirewallListResponse]]:
        """
        List DNS Firewall clusters for an account

        Args:
          account_id: Identifier.

          page: Page number of paginated results

          per_page: Number of clusters per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dns_firewall", account_id=account_id),
            page=AsyncV4PagePaginationArray[DNSFirewallListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    dns_firewall_list_params.DNSFirewallListParams,
                ),
            ),
            model=DNSFirewallListResponse,
        )

    async def delete(
        self,
        dns_firewall_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSFirewallDeleteResponse]:
        """
        Delete a DNS Firewall cluster

        Args:
          account_id: Identifier.

          dns_firewall_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dns_firewall_id:
            raise ValueError(f"Expected a non-empty value for `dns_firewall_id` but received {dns_firewall_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/dns_firewall/{dns_firewall_id}",
                account_id=account_id,
                dns_firewall_id=dns_firewall_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSFirewallDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSFirewallDeleteResponse]], ResultWrapper[DNSFirewallDeleteResponse]),
        )

    async def edit(
        self,
        dns_firewall_id: str,
        *,
        account_id: str,
        attack_mitigation: Optional[AttackMitigationParam] | Omit = omit,
        deprecate_any_requests: bool | Omit = omit,
        ecs_fallback: bool | Omit = omit,
        maximum_cache_ttl: float | Omit = omit,
        minimum_cache_ttl: float | Omit = omit,
        name: str | Omit = omit,
        negative_cache_ttl: Optional[float] | Omit = omit,
        ratelimit: Optional[float] | Omit = omit,
        retries: float | Omit = omit,
        upstream_ips: SequenceNotStr[UpstreamIPs] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSFirewallEditResponse]:
        """
        Modify the configuration of a DNS Firewall cluster

        Args:
          account_id: Identifier.

          dns_firewall_id: Identifier.

          attack_mitigation: Attack mitigation settings

          deprecate_any_requests: Whether to refuse to answer queries for the ANY type

          ecs_fallback: Whether to forward client IP (resolver) subnet if no EDNS Client Subnet is sent

          maximum_cache_ttl: By default, Cloudflare attempts to cache responses for as long as indicated by
              the TTL received from upstream nameservers. This setting sets an upper bound on
              this duration. For caching purposes, higher TTLs will be decreased to the
              maximum value defined by this setting.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

          minimum_cache_ttl: By default, Cloudflare attempts to cache responses for as long as indicated by
              the TTL received from upstream nameservers. This setting sets a lower bound on
              this duration. For caching purposes, lower TTLs will be increased to the minimum
              value defined by this setting.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

              Note that, even with this setting, there is no guarantee that a response will be
              cached for at least the specified duration. Cached responses may be removed
              earlier for capacity or other operational reasons.

          name: DNS Firewall cluster name

          negative_cache_ttl: This setting controls how long DNS Firewall should cache negative responses
              (e.g., NXDOMAIN) from the upstream servers.

              This setting does not affect the TTL value in the DNS response Cloudflare
              returns to clients. Cloudflare will always forward the TTL value received from
              upstream nameservers.

          ratelimit: Ratelimit in queries per second per datacenter (applies to DNS queries sent to
              the upstream nameservers configured on the cluster)

          retries: Number of retries for fetching DNS responses from upstream nameservers (not
              counting the initial attempt)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dns_firewall_id:
            raise ValueError(f"Expected a non-empty value for `dns_firewall_id` but received {dns_firewall_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/dns_firewall/{dns_firewall_id}",
                account_id=account_id,
                dns_firewall_id=dns_firewall_id,
            ),
            body=await async_maybe_transform(
                {
                    "attack_mitigation": attack_mitigation,
                    "deprecate_any_requests": deprecate_any_requests,
                    "ecs_fallback": ecs_fallback,
                    "maximum_cache_ttl": maximum_cache_ttl,
                    "minimum_cache_ttl": minimum_cache_ttl,
                    "name": name,
                    "negative_cache_ttl": negative_cache_ttl,
                    "ratelimit": ratelimit,
                    "retries": retries,
                    "upstream_ips": upstream_ips,
                },
                dns_firewall_edit_params.DNSFirewallEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSFirewallEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSFirewallEditResponse]], ResultWrapper[DNSFirewallEditResponse]),
        )

    async def get(
        self,
        dns_firewall_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSFirewallGetResponse]:
        """
        Show a single DNS Firewall cluster for an account

        Args:
          account_id: Identifier.

          dns_firewall_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dns_firewall_id:
            raise ValueError(f"Expected a non-empty value for `dns_firewall_id` but received {dns_firewall_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/dns_firewall/{dns_firewall_id}",
                account_id=account_id,
                dns_firewall_id=dns_firewall_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSFirewallGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSFirewallGetResponse]], ResultWrapper[DNSFirewallGetResponse]),
        )


class DNSFirewallResourceWithRawResponse:
    def __init__(self, dns_firewall: DNSFirewallResource) -> None:
        self._dns_firewall = dns_firewall

        self.create = to_raw_response_wrapper(
            dns_firewall.create,
        )
        self.list = to_raw_response_wrapper(
            dns_firewall.list,
        )
        self.delete = to_raw_response_wrapper(
            dns_firewall.delete,
        )
        self.edit = to_raw_response_wrapper(
            dns_firewall.edit,
        )
        self.get = to_raw_response_wrapper(
            dns_firewall.get,
        )

    @cached_property
    def analytics(self) -> AnalyticsResourceWithRawResponse:
        return AnalyticsResourceWithRawResponse(self._dns_firewall.analytics)

    @cached_property
    def reverse_dns(self) -> ReverseDNSResourceWithRawResponse:
        return ReverseDNSResourceWithRawResponse(self._dns_firewall.reverse_dns)


class AsyncDNSFirewallResourceWithRawResponse:
    def __init__(self, dns_firewall: AsyncDNSFirewallResource) -> None:
        self._dns_firewall = dns_firewall

        self.create = async_to_raw_response_wrapper(
            dns_firewall.create,
        )
        self.list = async_to_raw_response_wrapper(
            dns_firewall.list,
        )
        self.delete = async_to_raw_response_wrapper(
            dns_firewall.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            dns_firewall.edit,
        )
        self.get = async_to_raw_response_wrapper(
            dns_firewall.get,
        )

    @cached_property
    def analytics(self) -> AsyncAnalyticsResourceWithRawResponse:
        return AsyncAnalyticsResourceWithRawResponse(self._dns_firewall.analytics)

    @cached_property
    def reverse_dns(self) -> AsyncReverseDNSResourceWithRawResponse:
        return AsyncReverseDNSResourceWithRawResponse(self._dns_firewall.reverse_dns)


class DNSFirewallResourceWithStreamingResponse:
    def __init__(self, dns_firewall: DNSFirewallResource) -> None:
        self._dns_firewall = dns_firewall

        self.create = to_streamed_response_wrapper(
            dns_firewall.create,
        )
        self.list = to_streamed_response_wrapper(
            dns_firewall.list,
        )
        self.delete = to_streamed_response_wrapper(
            dns_firewall.delete,
        )
        self.edit = to_streamed_response_wrapper(
            dns_firewall.edit,
        )
        self.get = to_streamed_response_wrapper(
            dns_firewall.get,
        )

    @cached_property
    def analytics(self) -> AnalyticsResourceWithStreamingResponse:
        return AnalyticsResourceWithStreamingResponse(self._dns_firewall.analytics)

    @cached_property
    def reverse_dns(self) -> ReverseDNSResourceWithStreamingResponse:
        return ReverseDNSResourceWithStreamingResponse(self._dns_firewall.reverse_dns)


class AsyncDNSFirewallResourceWithStreamingResponse:
    def __init__(self, dns_firewall: AsyncDNSFirewallResource) -> None:
        self._dns_firewall = dns_firewall

        self.create = async_to_streamed_response_wrapper(
            dns_firewall.create,
        )
        self.list = async_to_streamed_response_wrapper(
            dns_firewall.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            dns_firewall.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            dns_firewall.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            dns_firewall.get,
        )

    @cached_property
    def analytics(self) -> AsyncAnalyticsResourceWithStreamingResponse:
        return AsyncAnalyticsResourceWithStreamingResponse(self._dns_firewall.analytics)

    @cached_property
    def reverse_dns(self) -> AsyncReverseDNSResourceWithStreamingResponse:
        return AsyncReverseDNSResourceWithStreamingResponse(self._dns_firewall.reverse_dns)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["LOADocumentCreateParams"]


class LOADocumentCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier of a Cloudflare account."""

    loa_document: Required[str]
    """LOA document to upload."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SchemaCreateParams"]


class SchemaCreateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    kind: Required[Literal["openapi_v3"]]
    """The kind of the schema"""

    name: Required[str]
    """A human-readable name for the schema"""

    source: Required[str]
    """The raw schema, e.g., the OpenAPI schema, either as JSON or YAML"""

    validation_enabled: Required[bool]
    """An indicator if this schema is enabled"""
